/*
MIDIATOR v3.5 - 5/23/2008

Copyright 2008 Curious Inventor, LLC
Creative Commons Attribution-Noncommercial 3.0
http://creativecommons.org/licenses/by-nc/3.0/
- you can modify and share, but please leave a link back to our
site: CuriousInventor.com , and seek permission before using this code
for commercial purposes. 

email: support@CuriousInventor.com or post on our forums

*/


//------------ CONFIG ------------------------------- //

// Configure the chip in code instead of through MPLAB's Configure--> Configuration Bits... 
// menu.

// Comment one of these out
#define CPU_FREQ_4MHz 
//#define CPU_FREQ_8MHz // still having some issues on some controllers running at 8MHz or higher.

// FOSC and CPUDIV setup up the clock speed and oscillator settings.
// HS = high speed, more power is needed at higher frequencies
#pragma config FOSC = HS  // Fosc = 8MHz 
#ifdef CPU_FREQ_4MHz
#pragma config CPUDIV = OSC2_PLL3 //OSC1_PLL2  // post scaler = 1 (this is a dual purpose setting, the PLL in the name implies that it does something in PLL mode, too)
#elif CPU_FREQ_8MHz
#pragma config CPUDIV = OSC1_PLL2
#endif

// Fosc = HS & CPUDIV = OSC2_PLL3 clocks cpu at 4MHz

//#pragma config FOSC = HSPLL_HS   // Fosc = 8MHz 
//#pragma config CPUDIV = OSC4_PLL6   // post scaler = 1 (this is a dual purpose setting, the PLL in the name implies that it does something in PLL mode, too)
#pragma config PLLDIV = 2
// CPUDIV = OSC4_PLL6 & PLLDIV = 2 clocks cpu at 16MHz.  OSC4_PLL6 means divide by 6 when using
// the PLL and divide by 4 when not using it.  The PLLDIV further divides by 2 when the PLL is 
// being used.
// So if I want a 16MHz Clock, do... OSC4_PLL6 and PLLDIV = 1

#pragma config BOR = OFF // brownout reset disabled
//#pragma config PBADEN = OFF // PORTB<4:0> configured as digital I/O on reset
#pragma config WDT = OFF // watch dog timer OFF






//------------ INCLUDES ------------------------------- //


#include <p18cxxx.h>
// automatically chooses the correct header file
#include <p18f4550.h>
#include "midiator.h"
#include "ps2.h"
#include "source/xlcd/xlcd.h"
#include <timers.h>
#include <usart.h>
#include "defs.h"
#include "source/xlcd/lcd.h"
#include "source/SW_UART/sw_uart.h"

#include <portb.h>
#include <delays.h>
#include <string.h>
#include <stdlib.h>
#include <spi.h>



//------------ DEFINES ------------------------------- //

// this is a shortcut for reversing the bit order of the 
// commands from the PS2 joysticks.
#define bitrev(c)	c = (c & 0x0F) << 4 | (c & 0xF0) >> 4; \
					c = (c & 0x33) << 2 | (c & 0xCC) >> 2; \
					c = (c & 0x55) << 1 | (c & 0xAA) >> 1;

#define USE_LCD
#define UP_ARROW 0
#define DOWN_ARROW 1
#define NUMBER_OF_LEDS 3
#define MIN_NOTE_SPACING 1
#define MIN_LENGTH 2

// state definitions
#define NOTE_TRIGGER_STATE 0
#define CHOOSE_BUTTON_STATE 1
#define CHOOSE_NOTE_STATE 2
#define MENU_TOP_STATE 3
#define RELATIVE_STATE 4 // experimental
#define QUASI_RELATIVE_STATE 5  // experimental
#define CHOOSE_MODE_STATE 6
#define CONTROLLER_STATE 7
#define CONTROLLER_TOGGLE_STATE 8
#define CONTROLLER_MISSING_STATE 9
#define PICK_CONTROLLER_STATE 10
#define CONFIG_JOYSTICK_STATE 11

// button mode defs
#define NOTE_TRIGGER_MODE 0
#define CONTROLLER_MODE 1
#define CONTROLLER_TOGGLE_MODE 2

#define MAX_ROW_STATE 1
#define MODE_MENU_MAX 2

#define ONE_TO_ONE_ROW_STATE 1
#define CHANGE_PLAY_MODE_ROW_STATE 0

#define X buttons[0]
#define Circle buttons[1]
#define Triangle buttons[2]
#define Square buttons[3]
#define Select buttons[4]
#define R1 buttons[5]
#define Left buttons[6]
#define Right buttons[7]
#define Up buttons[8]
#define Down buttons[9]
#define L1 buttons[10]
#define Start buttons[11]
#define L2 buttons[12]
#define R2 buttons[13]
#define L3 buttons[14]
#define R3 buttons[15]

#define JLX joysticks[0]
#define JLY joysticks[1]
#define JRX joysticks[2]
#define JRY joysticks[3]

#define SPEED_SCROLL_DELAY 1
#define SPEED_SCROLL_ENGAGE 10

#define CTRL_EEPROM_OFFSET 32

#define EEPROM_J_MIN 0
#define EEPROM_J_MAX 1
#define EEPROM_J_MID 2
#define EEPROM_J_CTRL 3
#define EEPROM_J_STEP 4
#define EEPROM_J_START 60




//------------ Prototypes ------------------------------- //

void Initial(void);
void initialize_buttons(void);
void respond_to_button(char PS2_button, char PS2_prev_button, button * cur_button);
void choose_button(char PS2_prev_button, button * cur_button);
void goto_state(char s);
void update_choose_note(char n);
void update_choose_controller(char n);
void update_button_mode(void);

void play_note(char num, char on_off);
void midi_controller_msg(char channel, char value);
void midi_pitch_bend(char value);

void display_button_name(char b);
void display_menu_top(void);
void display_number(char n);
void display_mode_menu(void);
void eeprom_write(char d, char addr);
char eeprom_read(char addr);
void delay_uS (char d);
void delay_mS (short long d);
void DelayFor18TCY(void);
void DelayPORXLCD(void);
void DelayXLCD(void);
extern void lcd_clear(void);
extern void lcd_gotoxy(char, char);
extern void lcd_define_char (char c, const char *bitmap);	
// LCD Display Functions //
void timer_isr (void);
void display_string(const rom char * text);
void display_string_mem(char * text);

void choose_joystick(void);
void update_joystick(void);

void display_cur_stick(void);



//------------ Global Variables ------------------------------- //

extern uint8 PS2_byte1, PS2_byte2, PS2_byte3, PS2_byte4, PS2_byte5, PS2_byte6;
extern uint8 PS2_prev_byte1, PS2_prev_byte2, PS2_prev_byte3, PS2_prev_byte4, PS2_prev_byte5, PS2_prev_byte6;
//extern uint8 PS2_byte7, PS2_byte8, PS2_byte9, PS2_byte10, PS2_byte11, PS2_byte12, PS2_byte13, PS2_byte14;
//extern uint8 PS2_byte15, PS2_byte16, PS2_byte17, PS2_byte18;
extern char PS2_pressure[12];
extern uint8 PS2_ID;

unsigned int DELAY;
char last_play_state;
char state;
char prev_state;
char menu_cur_button;
char menu_cur_mode;
unsigned char menu_cur_note;
unsigned char menu_cur_controller;
char menu_cur_stick;
char menu_cur_stick_prop;
char menu_top_row_state;
char current_pitch = 64;
char mode_menu_state;
char previous_note=0;
unsigned char led = 0;
button * get_button(char b);

uint8 SNESH;
uint8 SNESL;
uint8 SNESH_prev;
uint8 SNESL_prev;

button buttons[16];
joystick joysticks[4];

char button_hold_time = 0;
char direction = 5;
char byte9 = 0x01;

// this tells the chip where to go when an interrupt is triggered.
// the function below gets called.
#pragma code low_vector=0x18
void low_interrupt (void) 	{
	_asm GOTO timer_isr _endasm
}

// -------------  MAIN INTERRUPT FUNCTION ----------------//
// this gets called every time timer 1 throws an interrupt.  Each
// cycle, the PS2 is polled to see what buttons are pressed, internal
// states are changed, and MIDI is sent out.

#pragma code
#pragma interruptlow timer_isr
void timer_isr (void) {	
	char temp_buf[4];
	unsigned char p;
	unsigned char joyLX, joyLY, joyRX, joyRY;
	unsigned char joyLXin = 60;
	unsigned char joyLYin = 60;
	unsigned char joyRXin = 60;
	unsigned char joyRYin = 60;
  	char temp_joy_signed;
	unsigned char temp;
	int i;
	
	

	
	// see what buttons are pressed and where the joysticks are
	PS2_Talk();		
	
	//lcd_gotoxy(1,1);
	//display_string("hi");
	// toggle the SIGN-of-LIFE LED every 30 cycles
	if (led == 15) {
		LATEbits.LATE2 = LATEbits.LATE2 ^ 1;
		led=0;	
		//play_note(60,65);
	}	
	else
		led= led+1;
	
	// don't process buttons if the controller is missing or if it was just missing	
	if (state != CONTROLLER_MISSING_STATE && prev_state != CONTROLLER_MISSING_STATE) {
		
		// ---  JOYSTICK CONTROLLER MESSAGES --- //	
	  if (PS2_LJX != PS2_prev_LJX) {  // joystick has moved, send out a controller message
			joyLX = PS2_LJX;// change endianness (reverse the bits)	
			bitrev(joyLX);
			joyLXin = joyLX; // a 0 to 255 number
			if (joyLX <= 127) {
				joyLX = (unsigned char)((joyLX*JLX.mid_min)/(int)127 + JLX.min);
				//joyLX = (unsigned char)((joyLX*(int)20)/(int)127 + (int)15);
			}
			else {
				joyLX = (unsigned char)(((joyLX-127)*JLX.max_mid)/(int)128 + JLX.mid);
			}
			
			midi_controller_msg(JLX.controller_num,joyLX);				  		
		}
		if (PS2_LJY != PS2_prev_LJY) {  // joystick has moved, send out a controller message
			joyLY = PS2_LJY;// change endianness	
			bitrev(joyLY);
			joyLYin = joyLY;
			temp_joy_signed = joyLY/1; // the y axes come in backwards, temp_joy_signed helps flip them
			joyLY = -temp_joy_signed + 255;
			
			if (joyLY <= 127) {
				joyLY = (unsigned char)((joyLY*JLY.mid_min)/(int)127 + JLY.min);				
			}
			else {
				joyLY = (unsigned char)(((joyLY-127)*JLY.max_mid)/(int)128 + JLY.mid);
			}		
			midi_controller_msg(JLY.controller_num,joyLY);		 // modulation is channel 1	  		
		}
		if (PS2_RJX != PS2_prev_RJX) {  // joystick has moved, send out a controller message
			joyRX = PS2_RJX;// change endianness	
			bitrev(joyRX);
			joyRXin = joyRX;			
			if (joyRX <= 127) {
				joyRX = (unsigned char)((joyRX*JRX.mid_min)/(int)127 + JRX.min);				
			}
			else {
				joyRX = (unsigned char)(((joyRX-127)*JRX.max_mid)/(int)128 + JRX.mid);
			}		
			midi_controller_msg(JRX.controller_num,joyRX);			  		
		}
		if (PS2_RJY != PS2_prev_RJY) {  // joystick has moved, send out a controller message
			joyRY = PS2_RJY;// change endianness	
			bitrev(joyRY);
			joyRYin = joyRY;
			temp_joy_signed = joyRY/2;
			temp_joy_signed = -temp_joy_signed + 127;			
			midi_pitch_bend(temp_joy_signed);	  		
		}
		
		
		
		// The following code checks a variety of states to decide what to do for 
		// a given controller state (buttons pressed / unpressed).  The first state
		// to check is the general mode 'state'.  After that, the individual
		// buttons' states are checked to see if they were previously pressed and
		// for how long.  
		
		// If the controller was just checked to see if a button was pressed,
		// notes would continously re-trigger.  So we need to know what state
		// the button was in during the last cycle.  It's also possible (especially
		// on cheap, knock-off controllers) to double trigger a note or accidentally
		// play an extremely short note because the button wiggles slightly when 
		// pressed.  
	
		// To get around this, there is a minimum note length and time between
		// permissible note triggers.  Each button has a struct, 'button', (see
		// midiator.h) that keeps track of its current play-state (playing / not
		// playing), how long it's been held and how long it's been since last played.
		// All these states are used to decide what to do for a given up / down state
		// for each button.
		
		if (state == NOTE_TRIGGER_STATE || state == RELATIVE_STATE || state == QUASI_RELATIVE_STATE || state == CONTROLLER_STATE) {
				
			respond_to_button(PS2_X, PS2_prev_X, &X);
			respond_to_button(PS2_Circle, PS2_prev_Circle, &Circle);
			respond_to_button(PS2_Triangle,PS2_prev_Triangle,&Triangle);
			respond_to_button(PS2_Square, PS2_prev_Square, &Square);
			respond_to_button(PS2_Left, PS2_prev_Left, &Left);
			respond_to_button(PS2_Right, PS2_prev_Right, &Right);
			respond_to_button(PS2_Up, PS2_prev_Up, &Up);
			respond_to_button(PS2_Down, PS2_prev_Down, &Down);
			respond_to_button(PS2_L1, PS2_prev_L1, &L1);
			respond_to_button(PS2_R1, PS2_prev_R1, &R1);
			respond_to_button(PS2_L2, PS2_prev_L2, &L2);
			respond_to_button(PS2_R2, PS2_prev_R2, &R2);
			respond_to_button(PS2_L3, PS2_prev_L3, &L3);
			respond_to_button(PS2_R3, PS2_prev_R3, &R3);
			respond_to_button(PS2_Select, PS2_prev_Select, &Select);
		
			
			// the start button always goes in and out of the menu system.
			if (PS2_Start == 0) { // button is down
				if (PS2_prev_Start) { // button was just up, so it's a press
					// switch out of play state to MENU_TOP_STATE
					goto_state(CHOOSE_BUTTON_STATE);
				
				}						  	
			}
			else { 	
				if (Start.play_state == 1 && Start.min_length_timer == 0) {
					play_note(Start.note_num,0); // note off signal
					Start.play_state = 0; // play state is 0 now, so it won't continually send note-offs.			
				}			
			}	
	
		} 
		
			
		//------------   MENU SYSTEM  ---------------------------//
		
		else { // menu system:
			
			// the up and down arrows are used to cycle through choices,
			// x is the OK button, and circle is cancel / back.		
			if (PS2_Down == 0) { // button is down
				if (PS2_prev_Down) { // button was just up, so it's a press	
					if (state == MENU_TOP_STATE) {
						if (menu_top_row_state == CHOOSE_BUTTON_STATE)
							menu_top_row_state = CHOOSE_MODE_STATE;
						else if (menu_top_row_state == CHOOSE_MODE_STATE)
							menu_top_row_state = CHOOSE_BUTTON_STATE;
						display_menu_top();
					}
					else if (state == CHOOSE_MODE_STATE) {
						if (mode_menu_state == NOTE_TRIGGER_STATE) 
							mode_menu_state = CONTROLLER_TOGGLE_STATE;
						else if (mode_menu_state == CONTROLLER_TOGGLE_STATE)
							mode_menu_state = CONTROLLER_STATE;
					//	else if (mode_menu_state == RELATIVE_STATE) 
					//		mode_menu_state = QUASI_RELATIVE_STATE;
					//	else if (mode_menu_state == QUASI_RELATIVE_STATE)
					//		mode_menu_state = CONTROLLER_STATE;		
					  	else // CONTROLLER_STATE
					  		mode_menu_state = NOTE_TRIGGER_STATE;			
					    display_mode_menu();
					}		
					else if (state == CHOOSE_BUTTON_STATE) {
						menu_cur_button = Down.index;
						menu_cur_mode = Down.mode;
						goto_state(CHOOSE_NOTE_STATE);
					}				
					else if (state == CHOOSE_NOTE_STATE) {
						if (menu_cur_mode == NOTE_TRIGGER_MODE) {
							play_note(menu_cur_note, 0); // silence last note	
							if (menu_cur_note > 0) {
								menu_cur_note--;
								update_choose_note(menu_cur_note);
					  			play_note(menu_cur_note, 80);
							}
					  		else { // already at 0, next step will switch mode
					  			menu_cur_mode = CONTROLLER_TOGGLE_MODE;	
								update_button_mode();
							}				  			
						}
						else if (menu_cur_mode == CONTROLLER_MODE) {
							menu_cur_mode = NOTE_TRIGGER_MODE;
							menu_cur_note = 127;
							update_choose_note(menu_cur_note);
							play_note(menu_cur_note, 80);
						}
						else if (menu_cur_mode == CONTROLLER_TOGGLE_MODE) {
							menu_cur_mode = CONTROLLER_MODE;
							update_button_mode();
						}	
									  	
					}
					else if (state == CONFIG_JOYSTICK_STATE) {
						if (menu_cur_note > 0) {
							menu_cur_note--;
						}
				  	else { // already at 0, next step will switch mode
				  		menu_cur_note = 127;
						}		
						update_joystick();
					}
					button_hold_time = 0;
				}
				else { // button is being held down
					if (state == CHOOSE_NOTE_STATE)
						if (button_hold_time > SPEED_SCROLL_ENGAGE) {
						  play_note(menu_cur_note, 0);
							if (menu_cur_note > 0)
								menu_cur_note--;
					  //	else
					  //		menu_cur_note=127;
					  	update_choose_note(menu_cur_note);
					  	play_note(menu_cur_note, 80);	
					  	delay_mS(SPEED_SCROLL_DELAY);	
						}
						else
							button_hold_time++;
					else if (state == CONFIG_JOYSTICK_STATE) {
						if (button_hold_time > SPEED_SCROLL_ENGAGE) {						
							if (menu_cur_note > 0) {
								menu_cur_note--;					  	
					  	  update_joystick();					  
					  	  delay_mS(SPEED_SCROLL_DELAY);	
					  	}
					  	else {
						  	menu_cur_note = 127;
						  	update_joystick();
						  	delay_mS(SPEED_SCROLL_DELAY);	
						  }					  			
						}
						else
							button_hold_time++;
				  }    
				}
			}
			
		 // UP
		 else if(PS2_Up == 0) { // button is down
				if (PS2_prev_Up) { // button was just up, so it's a press		
					if (state == MENU_TOP_STATE) {
						if (menu_top_row_state == CHOOSE_BUTTON_STATE)
							menu_top_row_state = CHOOSE_MODE_STATE;
						else if (menu_top_row_state == CHOOSE_MODE_STATE)
							menu_top_row_state = CHOOSE_BUTTON_STATE;
						
						display_menu_top();
					}
					else if (state == CHOOSE_MODE_STATE) {
						if (mode_menu_state == NOTE_TRIGGER_STATE) 
							mode_menu_state = CONTROLLER_STATE;
					//	else if (mode_menu_state == RELATIVE_STATE) 
					//		mode_menu_state = NOTE_TRIGGER_STATE;
					//	else if (mode_menu_state == QUASI_RELATIVE_STATE)
					//		mode_menu_state = RELATIVE_STATE;	
					//	else
					//		mode_menu_state = QUASI_RELATIVE_STATE;	
						else if (mode_menu_state == CONTROLLER_STATE)
							mode_menu_state = CONTROLLER_TOGGLE_STATE;
						else
							mode_menu_state = NOTE_TRIGGER_STATE;
										
						display_mode_menu();
					}		
					else if (state == CHOOSE_BUTTON_STATE) {
						menu_cur_button = Up.index;
						goto_state(CHOOSE_NOTE_STATE);
					}
					else if (state == CHOOSE_NOTE_STATE) {
						if (menu_cur_mode == NOTE_TRIGGER_MODE) {
							play_note(menu_cur_note, 0); // silence last note	
							if (menu_cur_note < 127) {
								menu_cur_note++;
								update_choose_note(menu_cur_note);
					  			play_note(menu_cur_note, 80);
							}
					  		else { // already at 127, next step will switch mode
					  			menu_cur_mode = CONTROLLER_MODE;	
								update_button_mode();
							}				  			
						}
						else if (menu_cur_mode == CONTROLLER_TOGGLE_MODE) {
							menu_cur_mode = NOTE_TRIGGER_MODE;
							menu_cur_note = 0;
							update_choose_note(menu_cur_note);
							play_note(menu_cur_note, 80);
						}
						else if (menu_cur_mode == CONTROLLER_MODE) {
							menu_cur_mode = CONTROLLER_TOGGLE_MODE;
							update_button_mode();
						}	

					 		  	
					}
					else if (state == CONFIG_JOYSTICK_STATE) {
						if (menu_cur_note < 127) {
							menu_cur_note++;
						}
				  	else { // already at 127, next step will switch mode
				  		menu_cur_note = 0;
						}		
						update_joystick();
					}
					button_hold_time = 0;
				}
				else { // button is being held down
					if (state == CHOOSE_NOTE_STATE)
						if (button_hold_time > SPEED_SCROLL_ENGAGE) {
							play_note(menu_cur_note, 0);	
							if (menu_cur_note < 127)
								menu_cur_note++;
					  	//else
					  	//	menu_cur_note=0;
					  	update_choose_note(menu_cur_note);
					  	play_note(menu_cur_note, 80);	
					  	delay_mS(SPEED_SCROLL_DELAY);	
						}
						else
							button_hold_time++;
					else if (state == CONFIG_JOYSTICK_STATE) {
						if (button_hold_time > SPEED_SCROLL_ENGAGE) {						
							if (menu_cur_note < 127) {
								menu_cur_note++;					  	
					  	  update_joystick();					  
					  	  delay_mS(SPEED_SCROLL_DELAY);	
					  	}
					  	else {
						  	menu_cur_note = 0;
						  	update_joystick();
						  	delay_mS(SPEED_SCROLL_DELAY);	
						  }					  			
						}
						else
							button_hold_time++;
				  }  
				}				
			}
			
			else if(PS2_Triangle == 0) { // button is down
				choose_button(PS2_prev_Triangle, &Triangle);
			}		
			
			else if(PS2_Right == 0 && PS2_Up==1 && PS2_Down==1) { // button is down	
				if (PS2_prev_Right) { // it's a new press, not a continually pressed button
					if (state == CHOOSE_BUTTON_STATE) {
						choose_button(PS2_prev_Right, &Right);
					}
						
					else if (state == CHOOSE_NOTE_STATE) {
						if (menu_cur_mode != NOTE_TRIGGER_MODE) {
							if (menu_cur_controller < 127) {
								menu_cur_controller++;
								update_choose_controller(menu_cur_controller);
						  			//play_note(menu_cur_note, 80);
							}
						  else { // already at 127, wrap to 0
						  	menu_cur_controller	= 0;
								update_choose_controller(menu_cur_controller);
							}				  			
															  	
						}
					}	
					else if (state == CONFIG_JOYSTICK_STATE) {
						if (menu_cur_controller < 127) {
							menu_cur_controller++;
							update_joystick();
					  			//play_note(menu_cur_note, 80);
						}
					  else { // already at 127, wrap to 0
					  	menu_cur_controller	= 0;
							update_joystick();
						}			  			
						
					}
					button_hold_time = 0;					
				}
				else { // button is being held down
					if (state == CHOOSE_NOTE_STATE) {
						if (menu_cur_mode != NOTE_TRIGGER_MODE) {
							if (button_hold_time > SPEED_SCROLL_ENGAGE) {
							  play_note(menu_cur_note, 0);
								if (menu_cur_controller < 127)
									menu_cur_controller++;
						  	else
						  		menu_cur_controller=0;
							 	update_choose_controller(menu_cur_controller);
						 		play_note(menu_cur_note, 80);	
						 		delay_mS(SPEED_SCROLL_DELAY);	
							}
							else
								button_hold_time++;
						}
					  
					}
				
					else if (state == CONFIG_JOYSTICK_STATE) {
						if (button_hold_time > SPEED_SCROLL_ENGAGE) {
							if (menu_cur_controller < 127) {
								menu_cur_controller++;
							
						  			//play_note(menu_cur_note, 80);
							}
						  else { // already at 127, wrap to 0
						  	menu_cur_controller	= 0;
								
							}	
							delay_mS(SPEED_SCROLL_DELAY);	
							update_joystick();
						}		
						else
							button_hold_time++;
					}
				}
		  }	
			
			
			else if(PS2_Select == 0) { // button is down			
				choose_button(PS2_prev_Select, &Select);
			}
			else if(PS2_Start == 0) { // button is down			
				if (PS2_prev_Start) { // button was just up, so it's a press				
					 goto_state(NOTE_TRIGGER_STATE);
				}
			}
			else if(PS2_L1 == 0) { // button is down
				choose_button(PS2_prev_L1, &L1);			
			}
			else if(PS2_R1 == 0) { // button is down
				choose_button(PS2_prev_R1, &R1);		
			}
			
			else if(PS2_L2 == 0) { // button is down
				choose_button(PS2_prev_L2, &L2);			
			}
			else if(PS2_R2 == 0) { // button is down
				choose_button(PS2_prev_R2, &R2);			
			}
			
			else if(PS2_L3 == 0) { // button is down
				if (state == CHOOSE_BUTTON_STATE) {
					choose_button(PS2_prev_L3, &L3);		
				}
				else if (state == CHOOSE_NOTE_STATE && PS2_prev_L3) {
					menu_cur_stick = JLX.index;
					menu_cur_stick_prop = EEPROM_J_MID;	
					menu_cur_controller = JLX.controller_num;		
					menu_cur_note = JLX.mid;			
					choose_joystick();	
				}
				else if (state == CONFIG_JOYSTICK_STATE && PS2_prev_L3) {
					menu_cur_stick = JLY.index;
					menu_cur_stick_prop = EEPROM_J_MID;	
					menu_cur_controller = JLY.controller_num;		
					menu_cur_note = JLY.mid;			
					choose_joystick();	
				}
			}
			else if(PS2_R3 == 0) { // button is down
				if (state == CHOOSE_BUTTON_STATE) {
					choose_button(PS2_prev_R3, &R3);			
				}
				else if (state == CHOOSE_NOTE_STATE && PS2_prev_R3) {
					menu_cur_stick = JRX.index;
					menu_cur_stick_prop = EEPROM_J_MID;	
					menu_cur_controller = JRX.controller_num;		
					menu_cur_note = JRX.mid;			
					choose_joystick();	
				}
				else if (state == CONFIG_JOYSTICK_STATE && PS2_prev_R3) {
					menu_cur_stick = JRY.index;
					menu_cur_stick_prop = EEPROM_J_MID;	
					menu_cur_controller = JRY.controller_num;		
					menu_cur_note = JRY.mid;			
					choose_joystick();	
				}
			}
			
			else if(joyLXin > 245) {
				if (state == CHOOSE_BUTTON_STATE) {
					menu_cur_stick = JLX.index;
					menu_cur_stick_prop = EEPROM_J_MAX;	
					menu_cur_controller = JLX.controller_num;		
					menu_cur_note = JLX.max;			
					choose_joystick();
					
				}
			}
			else if(joyLXin < 10) {
				if (state == CHOOSE_BUTTON_STATE) {
					menu_cur_stick = JLX.index;
					menu_cur_stick_prop = EEPROM_J_MIN;	
					menu_cur_controller = JLX.controller_num;
					menu_cur_note = JLX.min;
					choose_joystick();
					
				}
			}
			else if(joyLYin > 245) {
				if (state == CHOOSE_BUTTON_STATE) {
					menu_cur_stick = JLY.index;
					menu_cur_stick_prop = EEPROM_J_MIN;	
					menu_cur_controller = JLY.controller_num;
					menu_cur_note = JLY.min;
					choose_joystick();
				}
			}
			else if(joyLYin < 10) {
				if (state == CHOOSE_BUTTON_STATE) {
					menu_cur_stick = JLY.index;
					menu_cur_stick_prop = EEPROM_J_MAX;	
					menu_cur_controller = JLY.controller_num;
					menu_cur_note = JLY.max;
					choose_joystick();
				}
			}
		
			else if(joyRXin > 245) {
				if (state == CHOOSE_BUTTON_STATE) {
					menu_cur_stick = JRX.index;
					menu_cur_stick_prop = EEPROM_J_MAX;
					menu_cur_controller = JRX.controller_num;	
					menu_cur_note = JRX.max;
					choose_joystick();
				}
			}
			else if(joyRXin < 10) {
				if (state == CHOOSE_BUTTON_STATE) {
					menu_cur_stick = JRX.index;
					menu_cur_stick_prop = EEPROM_J_MIN;	
					menu_cur_controller = JRX.controller_num;
					menu_cur_note = JRX.min;
					choose_joystick();
				}
			}
			else if(joyRYin > 245) {
				if (state == CHOOSE_BUTTON_STATE) {
					menu_cur_stick = JRY.index;
					menu_cur_stick_prop = EEPROM_J_MIN;	
					menu_cur_controller = JRY.controller_num;
					menu_cur_note = JRY.min;
					choose_joystick();
				}
			}
			else if(joyRYin < 10) {
				if (state == CHOOSE_BUTTON_STATE) {
					menu_cur_stick = JRY.index;
					menu_cur_stick_prop = EEPROM_J_MAX;	
					menu_cur_controller = JRY.controller_num;
					menu_cur_note = JRY.max;
					choose_joystick();				
				}
			}
			
			
			// this is the main select button, or OK button
			else if(PS2_X == 0) { // button is down
				
				if (PS2_prev_X) { // button was just up, so it's a press
					LATEbits.LATE1 = LATEbits.LATE1 ^ 1;
					
					if (state == MENU_TOP_STATE) {
						if (menu_top_row_state == CHOOSE_MODE_STATE) {
							goto_state(CHOOSE_MODE_STATE);						
						}
						else if (menu_top_row_state == CHOOSE_BUTTON_STATE)
							goto_state(CHOOSE_BUTTON_STATE);					
					}
					else if (state == CHOOSE_MODE_STATE) {
						goto_state(mode_menu_state);	
					}
					else if (state == CHOOSE_BUTTON_STATE) {
						menu_cur_button = X.index;						
						goto_state(CHOOSE_NOTE_STATE);
					}

					else if (state == CHOOSE_NOTE_STATE) {
						if (menu_cur_mode == NOTE_TRIGGER_MODE) {
							buttons[menu_cur_button].note_num = menu_cur_note;	
							buttons[menu_cur_button].mode = NOTE_TRIGGER_MODE;
							//buttons[menu_cur_button].controller_num = menu_cur_controller;
							//eeprom_write(buttons[menu_cur_button].controller_num, menu_cur_button+ CTRL_EEPROM_OFFSET);
							eeprom_write(buttons[menu_cur_button].note_num, menu_cur_button);				
							eeprom_write(NOTE_TRIGGER_MODE, menu_cur_button + 16);
							lcd_clear();
							lcd_gotoxy(1,1);
							display_string("saving...   ");
							//display_string("C");
							//display_number(menu_cur_controller);
							lcd_gotoxy(2,1);
							display_button_name(menu_cur_button);
							display_string(" --> ");
							display_number(menu_cur_note);
							play_note(menu_cur_note, 0);								
		  					delay_mS(130);		  				
							goto_state(CHOOSE_BUTTON_STATE);
						}
						else if (menu_cur_mode == CONTROLLER_MODE) {
							buttons[menu_cur_button].controller_num = menu_cur_controller;
							eeprom_write(buttons[menu_cur_button].controller_num, menu_cur_button+ CTRL_EEPROM_OFFSET);
							
							buttons[menu_cur_button].mode = CONTROLLER_MODE;
							eeprom_write(CONTROLLER_MODE, menu_cur_button + 16);
							lcd_clear();
							lcd_gotoxy(1,1);
							display_string("saving...");
							display_string("C");
							display_number(menu_cur_controller);
							lcd_gotoxy(2,1);
							display_button_name(menu_cur_button);
							display_string(" --> ");
							display_string("pressure");						
							delay_mS(130);		  				
							goto_state(CHOOSE_BUTTON_STATE);
						}
						else if (menu_cur_mode == CONTROLLER_TOGGLE_MODE) {
							buttons[menu_cur_button].controller_num = menu_cur_controller;
							eeprom_write(buttons[menu_cur_button].controller_num, menu_cur_button+ CTRL_EEPROM_OFFSET);
							
							buttons[menu_cur_button].mode = CONTROLLER_TOGGLE_MODE;
							eeprom_write(CONTROLLER_TOGGLE_MODE, menu_cur_button + 16);
							lcd_clear();
							lcd_gotoxy(1,1);
							display_string("saving...");
							display_string("C");
							display_number(menu_cur_controller);
							lcd_gotoxy(2,1);
							display_button_name(menu_cur_button);
							display_string(" --> ");
							display_string("ctrlTog ");
							
							delay_mS(130);		  				
							goto_state(CHOOSE_BUTTON_STATE);
						}
					}
					else if (state == CONFIG_JOYSTICK_STATE) {
						lcd_clear();
						lcd_gotoxy(1,1);
						display_string("saving...");	
						display_cur_stick();
						if (menu_cur_stick_prop == EEPROM_J_MIN) {
							joysticks[menu_cur_stick].min = menu_cur_note;							
							eeprom_write(menu_cur_note,EEPROM_J_START + joysticks[menu_cur_stick].index*EEPROM_J_STEP + EEPROM_J_MIN);
						}
						else if (menu_cur_stick_prop == EEPROM_J_MAX) {
							joysticks[menu_cur_stick].max = menu_cur_note;
							eeprom_write(menu_cur_note,EEPROM_J_START + joysticks[menu_cur_stick].index*EEPROM_J_STEP + EEPROM_J_MAX);
						}
						else if (menu_cur_stick_prop == EEPROM_J_MID){
							joysticks[menu_cur_stick].mid = menu_cur_note;
							eeprom_write(menu_cur_note,EEPROM_J_START + joysticks[menu_cur_stick].index*EEPROM_J_STEP + EEPROM_J_MID);
						}
					
						joysticks[menu_cur_stick].mid_min = joysticks[menu_cur_stick].mid-joysticks[menu_cur_stick].min;		
						joysticks[menu_cur_stick].max_mid = joysticks[menu_cur_stick].max-joysticks[menu_cur_stick].mid;
					
						joysticks[menu_cur_stick].controller_num = menu_cur_controller;						
						eeprom_write(menu_cur_controller,EEPROM_J_START + joysticks[menu_cur_stick].index*EEPROM_J_STEP + EEPROM_J_CTRL);
						lcd_gotoxy(2,1);
						display_string("value: ");						
						display_number(menu_cur_note);
						lcd_gotoxy(2,13);
						display_string("C");
						display_number(menu_cur_controller);
						
						delay_mS(130);		  				
						goto_state(CHOOSE_BUTTON_STATE);
						
					}
					
				}
			}
			
			// This is the main cancel / back button
			else if(PS2_Circle == 0) { // button is down
				if (PS2_prev_Circle) { // button was just up, so it's a press
					if (state == MENU_TOP_STATE) {
						goto_state(NOTE_TRIGGER_STATE);
					}
					else if (state == CHOOSE_BUTTON_STATE) {
						menu_cur_button = Circle.index;
						goto_state(CHOOSE_NOTE_STATE);
					}	 
					else if (state == CHOOSE_NOTE_STATE) {					
						goto_state(CHOOSE_BUTTON_STATE);				
					} 
				}
			}
			else if(PS2_Square == 0) { // button is down
				choose_button(PS2_prev_Square, &Square);			
			}	
			
			
			else if(PS2_Left == 0 && PS2_Up==1 && PS2_Down==1) { // button is down	
				if (PS2_prev_Left) { // it's a new press, not a continually pressed button
					if (state == CHOOSE_BUTTON_STATE) {
						choose_button(PS2_prev_Left, &Left);
					}
						
					else if (state == CHOOSE_NOTE_STATE) {
						if (menu_cur_mode != NOTE_TRIGGER_MODE) {
							if (menu_cur_controller > 0) {
								menu_cur_controller--;
								update_choose_controller(menu_cur_controller);
						  			//play_note(menu_cur_note, 80);
							}
						  else { // already at 0, wrap to 127
						  	menu_cur_controller	= 127;
								update_choose_controller(menu_cur_controller);
							}				  			
							
						}								  	
					}
					else if (state == CONFIG_JOYSTICK_STATE) {
						if (menu_cur_controller > 0) {
							menu_cur_controller--;
							update_joystick();
					  			//play_note(menu_cur_note, 80);
						}
					  else { // already at 127, wrap to 0
					  	menu_cur_controller	= 127;
							update_joystick();
						}			  			
						
					}	
					button_hold_time = 0;					
				}
				else { // button is being held down
					if (state == CHOOSE_NOTE_STATE) {
						if (menu_cur_mode != NOTE_TRIGGER_MODE) {
							if (button_hold_time > SPEED_SCROLL_ENGAGE) {
							  play_note(menu_cur_note, 0);
								if (menu_cur_controller > 0)
									menu_cur_controller--;
						  	else
						  		menu_cur_controller=127;
							 	update_choose_controller(menu_cur_controller);
						 		play_note(menu_cur_note, 80);	
						 		delay_mS(SPEED_SCROLL_DELAY);	
							}
							else
								button_hold_time++;
					  } 
					}
					else if (state == CONFIG_JOYSTICK_STATE) {
						if (button_hold_time > SPEED_SCROLL_ENGAGE) {
							if (menu_cur_controller > 0) {
								menu_cur_controller--;
							
						  			//play_note(menu_cur_note, 80);
							}
						  else { // already at 127, wrap to 0
						  	menu_cur_controller	= 127;
								
							}	
							delay_mS(SPEED_SCROLL_DELAY);	
							update_joystick();
						}		
						else
							button_hold_time++;
					}
				}
		  }	
		}	
		
		// decrement all the min time between notes values
		if (X.min_time_between_notes_timer > 0) X.min_time_between_notes_timer--;
		if (Circle.min_time_between_notes_timer > 0) Circle.min_time_between_notes_timer--;
		if (Triangle.min_time_between_notes_timer > 0) Triangle.min_time_between_notes_timer--;
		if (Square.min_time_between_notes_timer > 0) Square.min_time_between_notes_timer--;
		if (Up.min_time_between_notes_timer > 0) Up.min_time_between_notes_timer--;
		if (Down.min_time_between_notes_timer > 0) Down.min_time_between_notes_timer--;
		if (Left.min_time_between_notes_timer > 0) Left.min_time_between_notes_timer--;
		if (Right.min_time_between_notes_timer > 0) Right.min_time_between_notes_timer--;
		if (Select.min_time_between_notes_timer > 0) Select.min_time_between_notes_timer--;
		if (Start.min_time_between_notes_timer > 0) Start.min_time_between_notes_timer--;
		if (L1.min_time_between_notes_timer > 0) L1.min_time_between_notes_timer--;
		if (R1.min_time_between_notes_timer > 0) R1.min_time_between_notes_timer--;
		if (L2.min_time_between_notes_timer > 0) L2.min_time_between_notes_timer--;
		if (R2.min_time_between_notes_timer > 0) R2.min_time_between_notes_timer--;
		if (L3.min_time_between_notes_timer > 0) L3.min_time_between_notes_timer--;
		if (R3.min_time_between_notes_timer > 0) R3.min_time_between_notes_timer--;
		// decrement all the min length values
		if (X.min_length_timer > 0) X.min_length_timer--;
		if (Circle.min_length_timer > 0) Circle.min_length_timer--;
		if (Triangle.min_length_timer > 0) Triangle.min_length_timer--;
		if (Square.min_length_timer > 0) Square.min_length_timer--;
		if (Up.min_length_timer > 0) Up.min_length_timer--;
		if (Down.min_length_timer > 0) Down.min_length_timer--;
		if (Left.min_length_timer > 0) Left.min_length_timer--;
		if (Right.min_length_timer > 0) Right.min_length_timer--;
		if (Select.min_length_timer > 0) Select.min_length_timer--;
		if (Start.min_length_timer > 0) Start.min_length_timer--;
		if (L1.min_length_timer > 0) L1.min_length_timer--;
		if (R1.min_length_timer > 0) R1.min_length_timer--;
		if (L2.min_length_timer > 0) L2.min_length_timer--;
		if (R2.min_length_timer > 0) R2.min_length_timer--;
		if (L3.min_length_timer > 0) L3.min_length_timer--;
		if (R3.min_length_timer > 0) R3.min_length_timer--;
		
	
		
		PS2_prev_byte1 = PS2_byte1;
		PS2_prev_byte2 = PS2_byte2;
		PS2_prev_byte3 = PS2_byte3;
		PS2_prev_byte4 = PS2_byte4;
		PS2_prev_byte5 = PS2_byte5;
		PS2_prev_byte6 = PS2_byte6;
		// clear the interrupt so it can be triggered again
	}
	else if (prev_state == CONTROLLER_MISSING_STATE) {
		prev_state = state;
		PS2_prev_Start = 0;
	}

	INTCONbits.TMR0IF = 0;	
	
}

// Each note keeps track of whether it's on and how long it's been on to avoid
// double triggering and millisecond length notes (more problematic on cheap controllers).
// The code maintains a minimum time between successive notes and minimum note lengths.
// The controller mode makes all keys into on/ off switches for various controllers.
// The relative states were experiments in having each key increment or decrement instead of
// just playing a particular note.
void respond_to_button(char PS2_button, char PS2_prev_button, button * cur_button) {
	unsigned char temp;
	
	if (state == CONTROLLER_STATE && cur_button->index != 14 && cur_button->index != 15 && cur_button->index != 11 && cur_button->index != 4) {
		// also don't do this for select, start, R3 and L3	
		// send out a controller message for every single button every cycle (16 controller messages)
		//midi_controller_msg(cur_button->controller_num, PS2_pressure[cur_button->pressure_index]);
		
		// if it's not a guitar hero controller (no pressure), use the pressure for the controller value
	//	WriteUART(0x0D);
	//	print_ascii(PS2_ID.byte);
	//	bitrev(PS2_ID.byte);
		//print_ascii(PS2_ID.byte);
		//WriteUART(0x0D);
		//Delay1KTCYx(255);
		//Delay1KTCYx(255);
		if ((PS2_ID.byte & 0b00001111) == 0x09) {
			temp = PS2_pressure[cur_button->pressure_index];
			bitrev(temp);
			temp = temp >> 1; // chop off least significant bit
			midi_controller_msg(cur_button->controller_num, temp);
		}
		else { // no pressure signal, so just toggle 0 to 127
			if (PS2_button == 0) { // button down
				midi_controller_msg(cur_button->controller_num, 127);
			}
			else
				midi_controller_msg(cur_button->controller_num, 0);
		}	
	}		
	else {  // this is the only state available now
		if (cur_button->mode == NOTE_TRIGGER_MODE || cur_button->mode == CONTROLLER_TOGGLE_MODE) {		
			if (PS2_button == 0) { // button is down
				if (PS2_prev_button) { // button was just up, so it's a press			  
					if (cur_button->min_time_between_notes_timer == 0) { // it's been some time since the last press
						// ONE_TO_ONE is the simplest mode.  Each button is mapped to one note.
						if (state == NOTE_TRIGGER_STATE) {	// this is the meat of the current mode, since overall states can't be changed				
						
							if (cur_button->mode == NOTE_TRIGGER_MODE) {
								if (cur_button->play_state)
									play_note(cur_button->note_num,0); // some players count the number of note ons and offs,
									// if the prev note hasn't already been off'd, now is the time.					
								play_note(cur_button->note_num,cur_button->volume);
								cur_button->play_state = cur_button->note_num;
							}
							else if (cur_button->mode == CONTROLLER_TOGGLE_MODE) {
								if (cur_button->controller_value == 0) {
									cur_button->controller_value = 127;
									midi_controller_msg(cur_button->controller_num, 127);
								}
								else {
									cur_button->controller_value = 0;
									midi_controller_msg(cur_button->controller_num, 0);
								}
								
							}
						}
						// this is still in experimental mode.  The idea is that each button press
						// plays a new note some interval away.  So continued presses of Square might
						// play C, D, E, F#, etc.
						else if (state == RELATIVE_STATE) { // RELATIVE_STATE (EXPERIMENTAL)
							if (cur_button->play_state)
								play_note(cur_button->play_state,0); // some players count the number of note ons and offs,
								// if the prev note hasn't already been off'd, now is the time.					
							current_pitch += cur_button->relative_pitch;
							if (current_pitch > 127) current_pitch=127;
							if (current_pitch < 0) current_pitch=0;
							play_note(current_pitch,cur_button->volume);
							cur_button->play_state = current_pitch;
						}
						// Another experimental mode.  Here, some buttons shift the pitch that other buttons play.
						else if (state == QUASI_RELATIVE_STATE) { // EXPERIMENTAL
							if (cur_button->play_state)
								play_note(cur_button->play_state,0); 
							cur_button->play_state = cur_button->note_num + (SHIFT_RIGHT)*!PS2_Right + (SHIFT_UP)*(!PS2_Up) + (SHIFT_LEFT)*!PS2_Left + (SHIFT_DOWN)*!PS2_Down + (SHIFT_L)*!PS2_L1 + (SHIFT_R)*!PS2_R1;
							play_note(cur_button->play_state,cur_button->volume);		
							previous_note = cur_button->index;				
						}
						else if (state == CONTROLLER_STATE) {					
							midi_controller_msg(cur_button->controller_num, 127);						
							cur_button->play_state = 1;					
						}
						cur_button->min_length_timer = MIN_LENGTH;					
						cur_button->min_time_between_notes_timer = MIN_NOTE_SPACING; // bounce_loops before the button can be pressed again 	
					}				
				}		
			}
				
			else { // button is up
					// Here we can't just check to see if the button was JUST released,
					// because there's a chance that the initial release was ignored
					// because it came too soon after a press.  So we check to see if
					// the play_state is 1 and the min_play_length time has elapsed
					
				if (cur_button->play_state && cur_button->min_length_timer == 0) {
					if (state == CONTROLLER_STATE) {
						midi_controller_msg(cur_button->controller_num, 0);	
						//midi_controller_msg(25, 65);				  					
					}
					else {
						play_note(cur_button->play_state,0); // note off signal
					}
					cur_button->play_state = 0; // play state is 0 now, so it won't continually send note-offs.			
				}			
			}			
	
		}
		else if (cur_button->mode == CONTROLLER_MODE) {
			if ((PS2_ID.byte & 0b00001111) == 0x09) {
				temp = PS2_pressure[cur_button->pressure_index];
				bitrev(temp);
				temp = temp >> 1; // chop off least significant bit
				midi_controller_msg(cur_button->controller_num, temp);
			}
			else { // no pressure signal, so just toggle 0 to 127
				if (PS2_button == 0) { // button down
					midi_controller_msg(cur_button->controller_num, 127);
				}
				else
					midi_controller_msg(cur_button->controller_num, 0);
			}	
		}
	
	}
}



void choose_button(char PS2_prev_button, button * cur_button) {
	if (PS2_prev_button) { // button was just up, so it's a press				
		if (state == CHOOSE_BUTTON_STATE) {
			menu_cur_button = cur_button->index;
			goto_state(CHOOSE_NOTE_STATE);
		}
	}
}








/*-----------------------------------------------*/
/*------------   M A I N   -----------------------*/
/*-----------------------------------------------*/


void main (void) {	
	
	INTCONbits.GIE = 1;
	Initial(); 	
	LATEbits.LATE2 = 1;	
	
 	goto_state(NOTE_TRIGGER_STATE);
	//state =CONTROLLER_MISSING_STATE;
	
	OpenTimer0 (TIMER_INT_ON & T0_SOURCE_INT & T0_8BIT & T0_PS_1_64);    
	
	while (1) {	
	}
}



void Initial()
{
	const char const UpArrow[] = {0x04, 0x0e, 0x15, 0x04, 0x04, 0x04, 0x04, 0x04};
	const char const DownArrow[] = {0x04, 0x04, 0x04, 0x04, 0x04, 0x15, 0x0e, 0x04};

	ADCON0 = 0b00110000;

	// LCD INIT:
	// The arguments tell the LCD driver what size the LCD is and how the uC will
	// talk to it (4 bit vs 8 bit).
#ifdef USE_LCD
		OpenXLCD(FOUR_BIT & LINES_5X7); //0b00101000
		lcd_clear();
		//putsXLCD( mybuff );	
		
		delay_mS(10);

		lcd_define_char (UP_ARROW, UpArrow);	// Define user-defined char
		lcd_define_char (DOWN_ARROW, DownArrow);	// Define user-defined char	
		lcd_gotoxy(1,1);	
#endif
	initialize_buttons();
	/* ---------------  Software UART Config  ------------------- */
	// Note: Hardware USART and SDI overlap, so a software UART is
	// being used.    
	OpenUART(); 
  
  	/*
 	// if the hardware USART was being used, the following code would initialize
	TRISCbits.TRISC6 = 0; // makes TX an output  
 	// for sending MIDI out messages, 31250	 
  	OpenUSART( USART_TX_INT_OFF & 
			 USART_RX_INT_OFF &
			 USART_ASYNCH_MODE &
			 USART_EIGHT_BIT &
			 USART_SYNC_MASTER &
			 USART_BRGH_HIGH, 15);	// 7					  
  	// baud = Fosc / (16 (x + 1)), where x is the last arg to OpenUSART. 
  	*/  
    
	PS2_Init(); // sets B2 (PS2 ATT line) to output and configs SPI  
	
	// initialize LED ports and set their tris bits
	PORTEbits.RE1 = 0;
	PORTEbits.RE2 = 0;	
  TRISEbits.TRISE1 = 0;   
  TRISEbits.TRISE2 = 0; 
}

//------------  LCD HELPERS  ---------------- //

// different display funtions are called depending on whether the 
// characters to display are coming from rom or ram memory.
// for example: display('hi') instead of: my_var = 'hi', display(my_var).
void display_string(const rom char * text) {
	
		char temp[17];		
#ifdef USE_LCD
		strcpypgm2ram (temp, text);		
		putsXLCD(temp);			
#endif
}

void display_string_mem(char * text) {
#ifdef USE_LCD
		putsXLCD(text);	
#endif
}

void play_note(char num, char volume) {

	/* // if hardware USART:
	while (BusyUSART());
	putcUSART(0x90); // 9=note on, 0=channel0
	while (BusyUSART());
	putcUSART(num); // note number
	while (BusyUSART());		
	putcUSART(volume);	// volume 0x7F = 127 = max	
 	*/
 	
 	// Software UART is used because hardware is taken by SDI talking to the PS2
 	WriteUART(0x90);
	WriteUART(num);
	WriteUART(volume);
	LATEbits.LATE1 = LATEbits.LATE1 ^ 1;  	
}		

void midi_controller_msg(char channel, char value) {
	WriteUART(0xB0);
	WriteUART(channel); // controller number
	WriteUART(value); // controller value
}

void midi_pitch_bend(char value) {
	WriteUART(0xE0);
	WriteUART(0x00);
	WriteUART(value);
}


void initialize_buttons() {	
	char i= 0;
	char note;
	char mode;
	char controller;
	char min, max, mid, controller_num;

	for (i=0;i<16;i++) { // in case the eeprom has nothing in it, start each button off at 60+ and in NOTE_TRIGGER_MODE
		// note: eeprom writing doesn't work without general interrupts enabled.
		note = eeprom_read(i);
		mode = eeprom_read(i+16);
		controller = eeprom_read(i+CTRL_EEPROM_OFFSET);
	//	eeprom_write(0xFF,i);
	//	eeprom_write(0xFF,i+16);	
	
		if (note < 0 || note > 127) {
			eeprom_write(60+i,i);
		//	Delay100TCYx(200);
		}
		if (mode != NOTE_TRIGGER_MODE && mode != CONTROLLER_MODE && mode != CONTROLLER_TOGGLE_MODE) {
			eeprom_write(NOTE_TRIGGER_MODE,i+16);
		//	Delay100TCYx(200);
		}
		if (controller < 0 || controller > 127) {
			eeprom_write(10+i,i+CTRL_EEPROM_OFFSET);
		//	Delay100TCYx(200);
		}

	}
	//while(1);
	X.note_num = eeprom_read(0);
	X.volume = 127;
	X.play_state = 0;
	X.min_length_timer = 0;
	X.min_time_between_notes_timer = 0;
  	X.index = 0;
  	X.relative_pitch = 4;
  	X.controller_num = eeprom_read(0+CTRL_EEPROM_OFFSET);
	X.pressure_index = 6;
	X.mode = eeprom_read(16);
	X.controller_value = 0;

	Circle.note_num = eeprom_read(1);
	Circle.volume = 127;
	Circle.play_state = 0;
	Circle.min_length_timer = 0;
	Circle.min_time_between_notes_timer = 0;
	Circle.index = 1;
	Circle.relative_pitch = 0;
	Circle.controller_num = eeprom_read(1+CTRL_EEPROM_OFFSET);
	Circle.pressure_index = 5;
	Circle.mode = eeprom_read(1+16);
	Circle.controller_value = 0;
	
	Triangle.note_num = eeprom_read(2);
	Triangle.volume = 127;
	Triangle.play_state = 0;
	Triangle.min_length_timer = 0;
	Triangle.min_time_between_notes_timer = 0;
  	Triangle.index = 2;
  	Triangle.relative_pitch = 5;
	Triangle.controller_num = eeprom_read(2+CTRL_EEPROM_OFFSET);
	Triangle.pressure_index = 4;
	Triangle.mode = eeprom_read(2+16);
	Triangle.controller_value = 0;

	Square.note_num = eeprom_read(3);
	Square.volume = 127;
	Square.play_state = 0;
	Square.min_length_timer = 0;
	Square.min_time_between_notes_timer = 0;
 	Square.index = 3;
  	Square.relative_pitch = -4;
	Square.controller_num = eeprom_read(3+CTRL_EEPROM_OFFSET);
	Square.pressure_index = 7;
	Square.mode = eeprom_read(3+16);
	Square.controller_value = 0;

	Select.note_num = eeprom_read(4);
	Select.volume = 127;
	Select.play_state = 0;
	Select.min_length_timer = 0;
	Select.min_time_between_notes_timer = 0;
  	Select.index = 4;
  	Select.relative_pitch = 10;
	Select.controller_num = eeprom_read(4+CTRL_EEPROM_OFFSET);
	Select.mode = eeprom_read(4+16);
	Select.controller_value = 0;

	Start.note_num = eeprom_read(11);
	Start.volume = 127;
	Start.play_state = 0;
	Start.min_length_timer = 0;
	Start.min_time_between_notes_timer = 0;
  	Start.index = 11;
  	Start.controller_num = eeprom_read(11+CTRL_EEPROM_OFFSET);
	Start.mode = eeprom_read(11+16);
	Start.controller_value = 0;

	Left.note_num = eeprom_read(6);
	Left.volume = 127;
	Left.play_state = 0;
	Left.min_length_timer = 0;
	Left.min_time_between_notes_timer = 0;
  	Left.index = 6;
  	Left.relative_pitch = -1;
	Left.controller_num = eeprom_read(6+CTRL_EEPROM_OFFSET);
	Left.pressure_index = 1;
	Left.mode = eeprom_read(6+16);
	Left.controller_value = 0;

	Right.note_num = eeprom_read(7);
	Right.volume = 127;
	Right.play_state = 0;
	Right.min_length_timer = 0;
	Right.min_time_between_notes_timer = 0;
  	Right.index = 7;
  	Right.relative_pitch = 1;
	Right.controller_num = eeprom_read(7+CTRL_EEPROM_OFFSET);
	Right.pressure_index = 0;
	Right.mode = eeprom_read(7+16);
	Right.controller_value = 0;

	Up.note_num = eeprom_read(8);
	Up.volume = 127;
	Up.play_state = 0;
	Up.min_length_timer = 0;
	Up.min_time_between_notes_timer = 0;
  	Up.index = 8;
  	Up.relative_pitch = 2;
	Up.controller_num = eeprom_read(8+CTRL_EEPROM_OFFSET);
	Up.pressure_index = 2;
	Up.mode = eeprom_read(8+16);
	Up.controller_value = 0;

	Down.note_num = eeprom_read(9);
	Down.volume = 127;
	Down.play_state = 0;
	Down.min_length_timer = 0;
	Down.min_time_between_notes_timer = 0;
  	Down.index = 9;
	Down.relative_pitch = -2;
	Down.controller_num = eeprom_read(9+CTRL_EEPROM_OFFSET);
	Down.pressure_index = 3;
	Down.mode = eeprom_read(9+16);
	Down.controller_value = 0;

	L1.note_num = eeprom_read(10);
	L1.volume = 127;
	L1.play_state = 0;
	L1.min_length_timer = 0;
	L1.min_time_between_notes_timer = 0;
  	L1.index = 10;
  	L1.relative_pitch = 7;
	L1.controller_num = eeprom_read(10+CTRL_EEPROM_OFFSET);
	L1.pressure_index = 8;
	L1.mode = eeprom_read(10+16);
	L1.controller_value = 0;

	R1.note_num = eeprom_read(5);
	R1.volume = 127;
	R1.play_state = 0;
	R1.min_length_timer = 0;
	R1.min_time_between_notes_timer = 0;
  	R1.index = 5;
  	R1.relative_pitch = -7;
	R1.controller_num = eeprom_read(5+CTRL_EEPROM_OFFSET);
	R1.pressure_index = 9;
	R1.mode = eeprom_read(5+16);
	R1.controller_value = 0;

	L2.note_num = eeprom_read(12);
	L2.volume = 127;
	L2.play_state = 0;
	L2.min_length_timer = 0;
	L2.min_time_between_notes_timer = 0;
  	L2.index = 12;
  	L2.relative_pitch = 7;
	L2.controller_num = eeprom_read(12+CTRL_EEPROM_OFFSET);
	L2.pressure_index = 10;
	L2.mode = eeprom_read(12+16);
	L2.controller_value = 0;

	R2.note_num = eeprom_read(13);
	R2.volume = 127;
	R2.play_state = 0;
	R2.min_length_timer = 0;
	R2.min_time_between_notes_timer = 0;
  	R2.index = 13;
  	R2.relative_pitch = -7;
 	R2.controller_num = eeprom_read(13+CTRL_EEPROM_OFFSET);
	R2.pressure_index = 11;
    R2.mode = eeprom_read(13+16);
	R2.controller_value = 0;

  	L3.note_num = eeprom_read(14);
	L3.volume = 127;
	L3.play_state = 0;
	L3.min_length_timer = 0;
	L3.min_time_between_notes_timer = 0;
  	L3.index = 14;
  	L3.relative_pitch = 7;
	L3.controller_num = eeprom_read(14+CTRL_EEPROM_OFFSET);
	L3.mode = eeprom_read(14+16);
	L3.controller_value = 0;

	R3.note_num = eeprom_read(15);
	R3.volume = 127;
	R3.play_state = 0;
	R3.min_length_timer = 0;
	R3.min_time_between_notes_timer = 0;
  	R3.index = 15;
  	R3.relative_pitch = -7;
	R3.controller_num = eeprom_read(15+CTRL_EEPROM_OFFSET);
	R3.mode = eeprom_read(15+16);
	R3.controller_value = 0;
	
	
	// joystick data
	// each joystick, LX, LY, RX, RY has 4 properties:
	// min, mid, max and controller_num

	// default JRY to be pitch bend
	controller_num = eeprom_read(EEPROM_J_START + JRY.index*EEPROM_J_STEP + EEPROM_J_CTRL);
	if (controller_num < 0 || controller_num > 127) {
			eeprom_write(7,EEPROM_J_START + JRY.index*EEPROM_J_STEP + EEPROM_J_CTRL);		
	}
	controller_num = eeprom_read(EEPROM_J_START + JRX.index*EEPROM_J_STEP + EEPROM_J_CTRL);
	if (controller_num < 0 || controller_num > 127) {
			eeprom_write(31,EEPROM_J_START + JRX.index*EEPROM_J_STEP + EEPROM_J_CTRL);		
	}
	controller_num = eeprom_read(EEPROM_J_START + JLX.index*EEPROM_J_STEP + EEPROM_J_CTRL);
	if (controller_num < 0 || controller_num > 127) {
			eeprom_write(30,EEPROM_J_START + JLX.index*EEPROM_J_STEP + EEPROM_J_CTRL);		
	}
	
	// default JLY to be modulation
	controller_num = eeprom_read(EEPROM_J_START + JLY.index*EEPROM_J_STEP + EEPROM_J_CTRL);
	if (controller_num < 0 || controller_num > 127) {
			eeprom_write(1,EEPROM_J_START + JLY.index*EEPROM_J_STEP + EEPROM_J_CTRL);		
			// also set the mid to 0
			eeprom_write(0,EEPROM_J_START + JLY.index*EEPROM_J_STEP + EEPROM_J_MID);
			eeprom_write(127,EEPROM_J_START + JLY.index*EEPROM_J_STEP + EEPROM_J_MAX);
			eeprom_write(127,EEPROM_J_START + JLY.index*EEPROM_J_STEP + EEPROM_J_MIN);
	}
	
	
	 

	// load default data
	for (i=0;i<4;i++) { 
		min = eeprom_read(EEPROM_J_START + i*EEPROM_J_STEP + EEPROM_J_MIN);
		mid = eeprom_read(EEPROM_J_START + i*EEPROM_J_STEP + EEPROM_J_MID);
		max = eeprom_read(EEPROM_J_START + i*EEPROM_J_STEP + EEPROM_J_MAX);
		controller_num = eeprom_read(EEPROM_J_START + i*EEPROM_J_STEP + EEPROM_J_CTRL);
		
	
		if (min < 0 || min > 127) {
			min = 0;
			eeprom_write(min,EEPROM_J_START + i*EEPROM_J_STEP + EEPROM_J_MIN);		
		}
		if (mid < 0 || mid > 127) {
			mid = 63;
			eeprom_write(mid,EEPROM_J_START + i*EEPROM_J_STEP + EEPROM_J_MID);		
		}
		if (max < 0 || max > 127) {
			max = 127;
			eeprom_write(max,EEPROM_J_START + i*EEPROM_J_STEP + EEPROM_J_MAX);		
		}
	//	if (controller_num < 0 || controller_num > 127) {
	//		eeprom_write(64+i,EEPROM_J_START + i*EEPROM_J_STEP + EEPROM_J_CTRL);		
	//	}
		
		// load in values previously stored to the ram joysticks
		joysticks[i].min = min;
		joysticks[i].mid = mid;
		joysticks[i].max = max;
		joysticks[i].controller_num = controller_num;		
		joysticks[i].mid_min = mid-min;		
		joysticks[i].max_mid = max-mid;;
	
	
	}
	//#define JLX joysticks[0]
	//#define JLY joysticks[1]
	//#define JRX joysticks[2]
	//#define JRY joysticks[3]
	JLX.index = 0;
	JLY.index = 1;
	JRX.index = 2;
	JRY.index = 3;

	
	
	
}



void display_button_name(char b) {
#ifdef USE_LCD
	switch (b) {
		case 0:
			display_string("X");
			break;
		case 1:
			display_string("Circle");
			break;
	  case 2:
			display_string("Triangle");
			break;
		case 3:
			display_string("Square");
			break;
		case 4:
			display_string("Select");
			break;
		case 5:
			display_string("R1");
			break;	
		case 6:
			display_string("Left");
			break;			
		case 7:
			display_string("Right");
			break;
		case 8:
			display_string("Up");
			break;
		case 9:
			display_string("Down");
			break;
		case 10:
			display_string("L1");
			break;
		case 11:
			display_string("Start");
			break;	
		case 12:
			display_string("L2");
			break;
		case 13:
			display_string("R2");
			break;
		case 14:
			display_string("L3");
			break;
		case 15:
			display_string("R3");
			break;
						
		
	}	
#endif
}




void goto_state(char s) {
	state = s;	
#ifdef USE_LCD	
		lcd_clear();
		lcd_gotoxy(1,1);
#endif
	if (s==NOTE_TRIGGER_STATE) {
			display_string("MIDIATOR      v4");
			lcd_gotoxy(2,1);
			display_string("Start to ReMap");		
	}
	else if (s==MENU_TOP_STATE) {
#ifdef USE_LCD
		display_string("menu ");
		WriteDataXLCD (UP_ARROW);
		WriteDataXLCD(DOWN_ARROW);
		display_string(" cancel:O");
		lcd_gotoxy(2,1);
		display_string("OK:X  exit:Start");
#endif
		menu_top_row_state = CHOOSE_BUTTON_STATE;
	}
	else if (s==CHOOSE_BUTTON_STATE) {
#ifdef USE_LCD
		display_string("Press button");
		lcd_gotoxy(2,1);
		display_string("to remap...");
#endif
	}


	else if (s==CHOOSE_NOTE_STATE) {
		menu_cur_controller = buttons[menu_cur_button].controller_num;
		if (buttons[menu_cur_button].mode == NOTE_TRIGGER_MODE) {
			
			menu_cur_note = buttons[menu_cur_button].note_num;
			menu_cur_mode = buttons[menu_cur_button].mode;
			update_choose_note(buttons[menu_cur_button].note_num);
			play_note(menu_cur_note, 80);
		}
		else if (buttons[menu_cur_button].mode == CONTROLLER_MODE) {
			menu_cur_mode = buttons[menu_cur_button].mode;
			update_button_mode();
		}
		else if (buttons[menu_cur_button].mode == CONTROLLER_TOGGLE_MODE) {
			menu_cur_mode = buttons[menu_cur_button].mode;
			update_button_mode();
		}
		
	}
	//else if (s==CONFIG_JOYSTICK_STATE) {
	//	menu_cur_controller = joysticks[menu_cur_button].controller_num;			
	//	menu_cur_note = buttons[menu_cur_button].note_num;		
	//	update_choose_note(buttons[menu_cur_button].note_num);
	//}
	else if (s==RELATIVE_STATE) {
#ifdef USE_LCD
		display_string("MIDIATOR");
		lcd_gotoxy(2,1);
		display_string("Relative Mode");	
#endif
	}
	else if (s==CHOOSE_MODE_STATE) {
		display_mode_menu();
	}
	else if (s==QUASI_RELATIVE_STATE) {
#ifdef USE_LCD 
		display_string("Quasi Relative M");
		lcd_gotoxy(2,1);
		display_string("(start for menu)");	
#endif
	}
	else if (s==CONTROLLER_STATE) {
#ifdef USE_LCD
		display_string("MIDIATOR Menu:St");
		lcd_gotoxy(2,1);
		display_string("Controller Mode");	
#endif
	}
	else if (s==CONTROLLER_TOGGLE_STATE) {
#ifdef USE_LCD
		display_string("MIDIATOR Menu:St");
		lcd_gotoxy(2,1);
		display_string("ControllerToggle");	
#endif
	}
}

// This function is used in the menu system when remapping notes for
// ONE-TO-ONE mode.

void update_choose_controller(char n) {
#ifdef USE_LCD	
	if (menu_cur_mode == NOTE_TRIGGER_MODE) {
		update_choose_note(menu_cur_note);
	}
	else if (menu_cur_mode == CONTROLLER_TOGGLE_MODE) {
		update_button_mode();
	}
	else if (menu_cur_mode == CONTROLLER_MODE) {
		update_button_mode();
	}
#endif	
}	



void choose_joystick() {
	
		state = CONFIG_JOYSTICK_STATE;		
		lcd_clear();
		lcd_gotoxy(1,1);	
		display_string("set jystk ");
		display_cur_stick();
	
		update_joystick();
		//display_string("Start to ReMap");	
}

void display_cur_stick() {
	  if (menu_cur_stick == JLX.index) {
			display_string("LX ");	
		}
		else if (menu_cur_stick == JLY.index) {
			display_string("LY ");	
		}
		else if (menu_cur_stick == JRX.index) {
			display_string("RX ");	
		}
		else if (menu_cur_stick == JRY.index) {
			display_string("RY ");	
		}
		
		if (menu_cur_stick_prop == EEPROM_J_MIN) {
			display_string("MIN");	
		}
		else if (menu_cur_stick_prop == EEPROM_J_MAX) {
			display_string("MAX");	
		}
		else if (menu_cur_stick_prop == EEPROM_J_MID) {
			display_string("MID");	
		}
}

void update_joystick() {
		lcd_gotoxy(2,1);
		display_string("                ");
		lcd_gotoxy(2,1);
		display_string("value: ");
		display_number(menu_cur_note);
		lcd_gotoxy(2,13);
		display_string("C");
		display_number(menu_cur_controller);
}

void update_choose_note(char n) {	
#ifdef USE_LCD	
	display_string("remap ");
	display_button_name(menu_cur_button);
	display_string(" to:");
	lcd_gotoxy(2,1);
	display_string("note: ");
	display_number(n);
	display_string("       ");
	//lcd_gotoxy(2,11);
	//display_string("C");
	//display_number(menu_cur_controller);	
	lcd_gotoxy(2,15);
	WriteDataXLCD(UP_ARROW);
	WriteDataXLCD(DOWN_ARROW);
#endif
	
}

void update_button_mode() {
	lcd_clear();
	lcd_gotoxy(1,1);
	display_string("remap ");
	display_button_name(menu_cur_button);
	display_string(" to:");
	lcd_gotoxy(2,1);
	
	if (menu_cur_mode == CONTROLLER_MODE) {
		display_string("pressure  <>C   ");
		lcd_gotoxy(2,14);		
		display_number(menu_cur_controller);
	}
	else if (menu_cur_mode == CONTROLLER_TOGGLE_MODE) {
		display_string("Toggle    <>C  ");
		lcd_gotoxy(2,14);	
		display_number(menu_cur_controller);
	}


}

// -------------  Display helpers ----------------- //

void display_menu_top() {
#ifdef USE_LCD
	lcd_clear();
	lcd_gotoxy(1,1);
	display_string("menu ");
	WriteDataXLCD(UP_ARROW);
	WriteDataXLCD(DOWN_ARROW);
	lcd_gotoxy(2,1);
	if (menu_top_row_state == CHOOSE_MODE_STATE)
		display_string("change play mode");
	else if (menu_top_row_state == CHOOSE_BUTTON_STATE)
		display_string("1-to-1 mappings");
#endif
	//display_number(menu_top_row_state);
}

void display_number(char n) {
#ifdef USE_LCD
	char numstring[4];
	itoa(n, numstring);	
	display_string_mem(numstring);
	display_string(" ");
#endif
}

void display_mode_menu() {
#ifdef USE_LCD
	lcd_clear();
	lcd_gotoxy(1,1);
	display_string("Choose Mode: ");
	WriteDataXLCD(UP_ARROW);
	WriteDataXLCD(DOWN_ARROW);
	lcd_gotoxy(2,1);
	
	if (mode_menu_state == NOTE_TRIGGER_STATE) 
		display_string("NoteTrigger Mode");
	else if (mode_menu_state == RELATIVE_STATE) 
		display_string("Relative mode");
	else if (mode_menu_state == QUASI_RELATIVE_STATE) 
		display_string("Quasi Relative M");
	else if (mode_menu_state == CONTROLLER_STATE)
		display_string("Controller Mode");
	else if (mode_menu_state == CONTROLLER_TOGGLE_STATE)
		display_string("ControllerToggle");
#endif
	
}

// ---------- EEPROM functions ------------------- //

void eeprom_write(char d, char addr) {
// because this function is called from within an interrupt routine, interrupts are
// already disabled.
	EECON1bits.EEPGD = 0;  /* WRITE step #1 */
  EECON1bits.WREN = 1;   /* WRITE step #2 */
  EEADR = addr;             /* WRITE step #3 */
  EEDATA = d;    /* data */
  EECON2 = 0x55;         /* WRITE step #5 */
  EECON2 = 0xaa;         /* WRITE step #6 */
  EECON1bits.WR = 1;     /* WRITE step #7 */
  while (!PIR2bits.EEIF) /* WRITE step #8 */
    ;
  PIR2bits.EEIF = 0;     /* WRITE step #9 */ 
  EECON1bits.WREN = 0; 
}

char eeprom_read(char addr) {
	EECON1bits.EEPGD = 0;  /* READ step #1 */
	EECON1bits.CFGS = 0;
  EEADR = addr;             /* READ step #2 */
  EECON1bits.RD = 1;     /* READ step #3 */
  //direction = EEDATA;    /* READ step #4 */
	return EEDATA;	
}







